@@ -1,5 +1,5 @@
 """
-Routines for accessing a remote URI as if it where
+Routines for accessing a remote URI as if it were
 a remote bag of tiddlers, for use in a recipe. The
 idea is that if the bag portion of a line in a recipe
 is a URI, then we'll get whatever is on the other end
@@ -35,33 +35,15 @@ HTTP = None
 WHITE_DOMAINS = None
 
 
-def recipe_change_hook(store, recipe):
-    """
-    When a recipe is put, if a bag is_remote, create a
-    tiddler in REMOTEURI_BAG.
-    """
-    for bag, _ in recipe.get_recipe():
-        if is_remote(store.environ, bag, whiteforce=True):
-            update_remoteuri_bag(store, bag)
-
-
-def update_remoteuri_bag(store, uri):
-    """
-    Create a tiddler in REMOTEURI_BAG to indicate it has been whitelisted.
-    """
-    key = _remotebag_key(store.environ, uri)
-    store.put(Tiddler(key, REMOTEURI_BAG))
-
-
-HOOKS['recipe']['put'].append(recipe_change_hook)
-
-
 def init(config):
     """
     Initialize the plugin: setting up necessary defaults and globals.
     """
+    global HTTP
     config['special_bag_detectors'].append(is_remote)
 
+    HOOKS['recipe']['put'].append(recipe_change_hook)
+
     if config.get('remotebag.use_memcache'):
         import memcache
         cache = memcache.Client(config.get('memcache_hosts',
@@ -72,7 +54,6 @@ def init(config):
             path = os.path.join(config.get('root_dir', ''), path)
         cache = path
 
-    global HTTP
     HTTP = httplib2.Http(cache)
 
     store = get_store(config)
@@ -81,93 +62,6 @@ def init(config):
     ensure_bag(REMOTEURI_BAG, store, policy_dict=policy)
 
 
-def is_remote(environ, uri, whiteforce=False):
-    """
-    Return the tool for retrieving remote if this is a remote bag.
-    Otherwise None.
-    """
-    if uri.startswith('http:') or uri.startswith('https:'):
-
-        if whiteforce or is_white(environ, uri):
-
-            def curry(environ, func):
-                def actor(bag):
-                    return func(environ, bag)
-                return actor
-
-            return (curry(environ, get_remote_tiddlers),
-                    curry(environ, get_remote_tiddler))
-
-    return None
-
-
-def is_white(environ, uri):
-    """
-    Return true if the URI passes a whitelist mechanism.
-    Otherwise raise a ForbiddedError.
-    """
-    global WHITE_DOMAINS
-    netloc = urlparse(uri)[1]
-
-    if not WHITE_DOMAINS:
-        whitelist = environ.get('tiddlyweb.config', {}).get(
-                'remotebag.white_domains', [])
-        patterns = []
-        for domain in whitelist:
-            pattern = domain.replace('.', r'\.')
-            patterns.append(pattern)
-        WHITE_DOMAINS = re.compile('(?:' + '|'.join(patterns) + ')$')
-
-    if WHITE_DOMAINS.search(netloc) or via_recipe(environ, uri):
-        return True
-    raise ForbiddenError('remote uri not accepted: %s' % uri)
-
-
-def _remotebag_key(environ, uri):
-    """
-    Generate a tiddler title to use as the key in the REMOTEURI_BAG.
-    """
-    server_host = environ.get('tiddlyweb.config', {}).get(
-            'server_host')['host']
-    return sha(uri + server_host).hexdigest()
-
-
-def via_recipe(environ, uri):
-    """
-    Return true if this uri has been used in a recipe somewhere.
-    """
-    store = environ['tiddlyweb.store']
-    key = _remotebag_key(environ, uri)
-    try:
-        tiddler = Tiddler(key, REMOTEURI_BAG)
-        store.get(tiddler)
-        return True
-    except StoreError:
-        raise ForbiddenError('remote uri not whitelisted: %s' % uri)
-
-
-def retrieve_remote(uri, accept=None, method='GET'):
-    """
-    Do an http request to get the remote content.
-    """
-    uri = uri.encode('UTF-8')
-    try:
-        if accept:
-            response, content = HTTP.request(uri, method=method,
-                    headers={'Accept': accept})
-        else:
-            response, content = HTTP.request(uri, method=method)
-    except httplib2.HttpLib2Error, exc:
-        raise SpecialBagError('unable to retrieve remote: %s: %s'
-                % (uri, exc))
-
-    if response['status'] == '200' or response['status'] == '304':
-        return response, content
-    else:
-        raise SpecialBagError('bad response from remote: %s: %s: %s'
-                % (uri, response['status'], content))
-
-
 def get_remote_tiddlers(environ, uri):
     """
     Retrieve the tiddlers at uri, yield as skinny tiddlers.
@@ -217,13 +111,156 @@ def get_remote_tiddler_html(environ, uri, title):
     return tiddler
 
 
-def _get_tiddlyweb_tiddler(environ, uri, title):
+def is_remote(environ, uri, whiteforce=False):
     """
-    Get a tiddler with title from uri.
+    Return the tool for retrieving remote if this is a remote bag.
+    Otherwise None.
     """
-    url = uri + '/' + encode_name(title)
-    _, content = retrieve_remote(url, accept='application/json')
-    return _process_json_tiddler(environ, content, uri)
+    if uri.startswith('http:') or uri.startswith('https:'):
+
+        if whiteforce or is_white(environ, uri):
+
+            def curry(environ, func):
+                """
+                Enclose the current environ for the remote retriever.
+                """
+                def actor(bag):
+                    return func(environ, bag)
+                return actor
+
+            return (curry(environ, get_remote_tiddlers),
+                    curry(environ, get_remote_tiddler))
+
+    return None
+
+
+def is_white(environ, uri):
+    """
+    Return true if the URI passes a whitelist mechanism.
+    Otherwise raise a ForbiddedError.
+    """
+    global WHITE_DOMAINS
+    netloc = urlparse(uri)[1]
+
+    if not WHITE_DOMAINS:
+        whitelist = environ.get('tiddlyweb.config', {}).get(
+                'remotebag.white_domains', [])
+        patterns = []
+        for domain in whitelist:
+            pattern = domain.replace('.', r'\.')
+            patterns.append(pattern)
+        WHITE_DOMAINS = re.compile('(?:' + '|'.join(patterns) + ')$')
+
+    if WHITE_DOMAINS.search(netloc) or via_recipe(environ, uri):
+        return True
+    raise ForbiddenError('remote uri not accepted: %s' % uri)
+
+
+def recipe_change_hook(store, recipe):
+    """
+    When a recipe is put, if a bag is_remote, create a
+    tiddler in REMOTEURI_BAG.
+    """
+    for bag, _ in recipe.get_recipe():
+        if is_remote(store.environ, bag, whiteforce=True):
+            update_remoteuri_bag(store, bag)
+
+
+def retrieve_remote(uri, accept=None, method='GET'):
+    """
+    Do an http request to get the remote content.
+    """
+    uri = uri.encode('UTF-8')
+    try:
+        if accept:
+            response, content = HTTP.request(uri, method=method,
+                    headers={'Accept': accept})
+        else:
+            response, content = HTTP.request(uri, method=method)
+    except httplib2.HttpLib2Error, exc:
+        raise SpecialBagError('unable to retrieve remote: %s: %s'
+                % (uri, exc))
+
+    if response['status'] == '200' or response['status'] == '304':
+        return response, content
+    else:
+        raise SpecialBagError('bad response from remote: %s: %s: %s'
+                % (uri, response['status'], content))
+
+
+def update_remoteuri_bag(store, uri):
+    """
+    Create a tiddler in REMOTEURI_BAG to indicate it has been whitelisted.
+    """
+    key = _remotebag_key(store.environ, uri)
+    store.put(Tiddler(key, REMOTEURI_BAG))
+
+
+def via_recipe(environ, uri):
+    """
+    Return true if this uri has been used in a recipe somewhere.
+    """
+    store = environ['tiddlyweb.store']
+    key = _remotebag_key(environ, uri)
+    try:
+        tiddler = Tiddler(key, REMOTEURI_BAG)
+        store.get(tiddler)
+        return True
+    except StoreError:
+        raise ForbiddenError('remote uri not whitelisted: %s' % uri)
+
+
+class RemoteTiddler(Tiddler):
+    """
+    A subclass of the standard Tiddler that allows text to be
+    a property. This makes it possible to only get the text
+    of a remote tiddler when it is actually asked for. This
+    avoids overzealous retrieval of single tiddlers when the
+    data in skinny collection of tiddlers will do okay.
+    """
+
+    def __init__(self, title=None, bag=None):
+        Tiddler.__init__(self, title, bag)
+        self._text = None
+
+    def get_text(self):
+        """
+        Retrieve tiddler text from the remote store only if
+        requested.
+        """
+        if self._text is None:
+            try:
+                self = self.store.get(self)
+                self._text = self.text
+            except (AttributeError, StoreError):
+                return ''
+        return self._text
+
+    def set_text(self, value):
+        self._text = value
+
+    def del_text(self):
+        self._text = None
+
+    text = property(get_text, set_text, del_text, "Manage text attribute")
+
+
+def _determine_remote_handler(environ, uri):
+    """
+    Determine which remote handler to use for this uri.
+    """
+    config = environ['tiddlyweb.config']
+    testers = [(_test_uri_for_tiddlers,
+        (_get_tiddlyweb_tiddlers, _get_tiddlyweb_tiddler))]
+
+    testers.extend(config.get('remotebag.remote_handlers', []))
+
+    for tester, target in testers:
+        if tester(environ, uri):
+            return target
+
+    # do default, getting raw html
+    return (get_remote_tiddlers_html, get_remote_tiddler_html)
 
 
 def _get_tiddlyweb_tiddlers(environ, uri):
@@ -234,41 +271,13 @@ def _get_tiddlyweb_tiddlers(environ, uri):
     return _process_json_tiddlers(environ, content, uri)
 
 
-def _test_uri_for_tiddlers(environ, uri):
+def _get_tiddlyweb_tiddler(environ, uri, title):
     """
-    Return true if uri looks like a bags or recipes tiddlers URI.
+    Get a tiddler with title from uri.
     """
-    return TIDDLERS_PATTERN.search(uri)
-
-
-TESTERS = [(_test_uri_for_tiddlers,
-    (_get_tiddlyweb_tiddlers, _get_tiddlyweb_tiddler))]
-
-
-def _determine_remote_handler(environ, uri):
-    """
-    Determine which remote handler to use for this uri.
-    """
-    config = environ['tiddlyweb.config']
-    if len(TESTERS) == 1:
-        TESTERS.extend(config.get('remotebag.remote_handlers', []))
-    for tester, target in TESTERS:
-        if tester(environ, uri):
-            return target
-    # do default, getting raw html
-    return (get_remote_tiddlers_html, get_remote_tiddler_html)
-
-
-def _process_json_tiddler(environ, content, uri):
-    """
-    Transmute JSON content into a Tiddler.
-    """
-    content = content.decode('utf-8')
-    data = simplejson.loads(content)
-    tiddler = RemoteTiddler(data['title'], uri)
-    serializer = Serializer('json', environ)
-    serializer.object = tiddler
-    return serializer.from_string(content)
+    url = uri + '/' + encode_name(title)
+    _, content = retrieve_remote(url, accept='application/json')
+    return _process_json_tiddler(environ, content, uri)
 
 
 def _process_json_tiddlers(environ, content, uri):
@@ -285,7 +294,7 @@ def _process_json_tiddlers(environ, content, uri):
     for item in data:
         tiddler = RemoteTiddler(item['title'], uri)
         for key in ['creator', 'fields', 'created', 'modified',
-                'modifier', 'type','tags']:
+                'modifier', 'type', 'tags']:
             try:
                 setattr(tiddler, key, item[key])
             except (KeyError, AttributeError):
@@ -293,25 +302,30 @@ def _process_json_tiddlers(environ, content, uri):
         tiddler.store = store
         yield tiddler
 
-class RemoteTiddler(Tiddler):
 
-    def __init__(self, title=None, bag=None):
-        Tiddler.__init__(self, title, bag)
-        self._text = None
+def _process_json_tiddler(environ, content, uri):
+    """
+    Transmute JSON content into a Tiddler.
+    """
+    content = content.decode('utf-8')
+    data = simplejson.loads(content)
+    tiddler = RemoteTiddler(data['title'], uri)
+    serializer = Serializer('json', environ)
+    serializer.object = tiddler
+    return serializer.from_string(content)
 
-    def get_text(self):
-        if self._text is None:
-            try:
-                self = self.store.get(self)
-                self._text = self.text
-            except (AttributeError, StoreError):
-                return ''
-        return self._text
 
-    def set_text(self, value):
-        self._text = value
+def _remotebag_key(environ, uri):
+    """
+    Generate a tiddler title to use as the key in the REMOTEURI_BAG.
+    """
+    server_host = environ.get('tiddlyweb.config', {}).get(
+            'server_host')['host']
+    return sha(uri + server_host).hexdigest()
 
-    def del_text(self):
-        self._text = None
 
-    text = property(get_text, set_text, del_text, "Manage text attribute")
+def _test_uri_for_tiddlers(environ, uri):
+    """
+    Return true if uri looks like a bags or recipes tiddlers URI.
+    """
+    return TIDDLERS_PATTERN.search(uri)
